@@ -1,4 +1,4 @@
-/*	$OpenBSD: def.h,v 1.157 2018/12/13 14:59:16 lum Exp $	*/
+/*	$OpenBSD: def.h,v 1.165 2019/07/18 10:55:11 lum Exp $	*/
 
 /* This file is in the public domain. */
 
@@ -51,7 +51,7 @@ typedef int	(*PF)(int, int);	/* generally useful type */
  * flag controls the clearing versus appending
  * of data in the kill buffer.
  */
-#define CFCPCN	0x0001		/* Last command was C-P, C-N	 */
+#define CFCPCN	0x0001		/* Last command was C-p or C-n	 */
 #define CFKILL	0x0002		/* Last command was a kill	 */
 #define CFINS	0x0004		/* Last command was self-insert	 */
 
@@ -103,6 +103,7 @@ typedef int	(*PF)(int, int);	/* generally useful type */
 
 #define MAX_TOKEN 64
 
+#define	BUFSIZE	128	/* Size of line contents in extend.c  */
 /*
  * Previously from sysdef.h
  */
@@ -448,12 +449,12 @@ int		 diffbuffer(int, int);
 struct buffer	*findbuffer(char *);
 
 /* display.c */
-int		vtresize(int, int, int);
-void		vtinit(void);
-void		vttidy(void);
-void		update(int);
-int		linenotoggle(int, int);
-int		colnotoggle(int, int);
+int		 vtresize(int, int, int);
+void		 vtinit(void);
+void		 vttidy(void);
+void		 update(int);
+int		 linenotoggle(int, int);
+int		 colnotoggle(int, int);
 
 /* echo.c X */
 void		 eerase(void);
@@ -471,7 +472,7 @@ int		 ffropen(FILE **, const char *, struct buffer *);
 void		 ffstat(FILE *, struct buffer *);
 int		 ffwopen(FILE **, const char *, struct buffer *);
 int		 ffclose(FILE *, struct buffer *);
-int		 ffputbuf(FILE *, struct buffer *);
+int		 ffputbuf(FILE *, struct buffer *, int);
 int		 ffgetline(FILE *, char *, int, int *);
 int		 fbackupfile(const char *);
 char		*adjustname(const char *, int);
@@ -495,6 +496,7 @@ int		 rescan(int, int);
 int		 universal_argument(int, int);
 int		 digit_argument(int, int);
 int		 negative_argument(int, int);
+int		 ask_selfinsert(int, int);
 int		 selfinsert(int, int);
 int		 quote(int, int);
 
@@ -531,7 +533,7 @@ int		 swapmark(int, int);
 int		 gotoline(int, int);
 int		 setlineno(int);
 
-/* random.c X */
+/* util.c X */
 int		 showcpos(int, int);
 int		 getcolpos(struct mgwin *);
 int		 twiddle(int, int);
@@ -557,19 +559,19 @@ int		 tagsvisit(int, int);
 int		 curtoken(int, int, char *);
 
 /* cscope.c */
-int		cssymbol(int, int);
-int		csdefinition(int, int);
-int		csfuncalled(int, int);
-int		cscallerfuncs(int, int);
-int		csfindtext(int, int);
-int		csegrep(int, int);
-int		csfindfile(int, int);
-int		csfindinc(int, int);
-int		csnextfile(int, int);
-int		csnextmatch(int, int);
-int		csprevfile(int, int);
-int		csprevmatch(int, int);
-int		cscreatelist(int, int);
+int		 cssymbol(int, int);
+int		 csdefinition(int, int);
+int		 csfuncalled(int, int);
+int		 cscallerfuncs(int, int);
+int		 csfindtext(int, int);
+int		 csegrep(int, int);
+int		 csfindfile(int, int);
+int		 csfindinc(int, int);
+int		 csnextfile(int, int);
+int		 csnextmatch(int, int);
+int		 csprevfile(int, int);
+int		 csprevmatch(int, int);
+int		 cscreatelist(int, int);
 
 /* extend.c X */
 int		 insert(int, int);
@@ -584,6 +586,7 @@ int		 evalbuffer(int, int);
 int		 evalfile(int, int);
 int		 load(const char *);
 int		 excline(char *);
+char		*skipwhite(char *);
 
 /* help.c X */
 int		 desckey(int, int);
@@ -715,8 +718,14 @@ int		 compile(int, int);
 void		 bellinit(void);
 int		 toggleaudiblebell(int, int);
 int		 togglevisiblebell(int, int);
+int		 dobeep_msgs(const char *, const char *);
+int		 dobeep_msg(const char *);
 void		 dobeep(void);
 
+/* interpreter.c */
+int		 foundparen(char *);
+int		 clearvars(void);
+
 /*
  * Externals.
  */
